@@ -116,12 +116,12 @@ atspi_component_get_accessible_at_point (AtspiComponent *obj,
                                           AtspiCoordType ctype, GError **error)
 {
   dbus_int32_t d_x = x, d_y = y;
-  dbus_uint16_t d_ctype = ctype;
+  dbus_uint32_t d_ctype = ctype;
   DBusMessage *reply;
 
   g_return_val_if_fail (obj != NULL, FALSE);
 
-  reply = _atspi_dbus_call_partial (obj, atspi_interface_component, "GetAccessibleAtPoint", error, "iin", d_x, d_y, d_ctype);
+  reply = _atspi_dbus_call_partial (obj, atspi_interface_component, "GetAccessibleAtPoint", error, "iiu", d_x, d_y, d_ctype);
 
   return _atspi_dbus_return_accessible_from_message (reply);
 }
@@ -310,7 +310,7 @@ atspi_component_set_extents (AtspiComponent *obj,
   dbus_uint32_t d_ctype = ctype;
   DBusMessageIter iter, iter_struct;
   DBusMessage *message, *reply;
-  dbus_bool_t retval;
+  dbus_bool_t retval = FALSE;
   AtspiAccessible *aobj = ATSPI_ACCESSIBLE (obj);
 
   g_return_val_if_fail (obj != NULL, FALSE);
