@@ -4,6 +4,7 @@
  *
  * Copyright 2002 Ximian Inc.
  * Copyright 2002 Sun Microsystems, Inc.
+ * Copyright 2010, 2011 Novell, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +24,7 @@
 
 #include "atspi-private.h"
 #include <string.h>
+#include <ctype.h>
 
 typedef struct
 {
@@ -62,7 +64,7 @@ typedef struct
 static GHashTable *callbacks;
 
 void
-callback_ref (void *callback, GDestroyNotify *callback_destroyed)
+callback_ref (void *callback, GDestroyNotify callback_destroyed)
 {
   CallbackInfo *info;
 
@@ -185,6 +187,9 @@ cache_process_children_changed (AtspiEvent *event)
   else if (g_list_find (event->source->children, child))
   {
     event->source->children = g_list_remove (event->source->children, child);
+    if (child == child->parent.app->root)
+      g_object_run_dispose (G_OBJECT (child->parent.app));
+    g_object_unref (child);
   }
 }
 
@@ -198,7 +203,7 @@ cache_process_property_change (AtspiEvent *event)
     if (G_VALUE_HOLDS (&event->any_data, ATSPI_TYPE_ACCESSIBLE))
     {
       event->source->accessible_parent = g_value_dup_object (&event->any_data);
-      event->source->cached_properties |= ATSPI_CACHE_PARENT;
+      _atspi_accessible_add_cache (event->source, ATSPI_CACHE_PARENT);
     }
     else
     {
@@ -213,7 +218,7 @@ cache_process_property_change (AtspiEvent *event)
     if (G_VALUE_HOLDS_STRING (&event->any_data))
     {
       event->source->name = g_value_dup_string (&event->any_data);
-      event->source->cached_properties |= ATSPI_CACHE_NAME;
+      _atspi_accessible_add_cache (event->source, ATSPI_CACHE_NAME);
     }
     else
     {
@@ -228,7 +233,7 @@ cache_process_property_change (AtspiEvent *event)
     if (G_VALUE_HOLDS_STRING (&event->any_data))
     {
       event->source->description = g_value_dup_string (&event->any_data);
-      event->source->cached_properties |= ATSPI_CACHE_DESCRIPTION;
+      _atspi_accessible_add_cache (event->source, ATSPI_CACHE_DESCRIPTION);
     }
     else
     {
@@ -307,7 +312,6 @@ convert_event_type_to_dbus (const char *eventType, char **categoryp, char **name
   gchar *tmp = strdup_and_adjust_for_dbus (eventType);
   char *category = NULL, *name = NULL, *detail = NULL;
   char *saveptr = NULL;
-  char *p;
 
   if (tmp == NULL) return FALSE;
   category = strtok_r (tmp, ":", &saveptr);
@@ -363,7 +367,7 @@ oom:
 static void
 listener_entry_free (EventListenerEntry *e)
 {
-  gpointer callback = (e->callback = remove_datum ? e->user_data : e->callback);
+  gpointer callback = (e->callback == remove_datum ? e->user_data : e->callback);
   g_free (e->category);
   g_free (e->name);
   if (e->detail) g_free (e->detail);
@@ -374,8 +378,6 @@ listener_entry_free (EventListenerEntry *e)
 /**
  * atspi_event_listener_register:
  * @listener: The #AtspiEventListener to register against an event type.
- * @user_data: (closure): User data to be passed to the callback.
- * @callback_destroyed: A #GDestroyNotify called when the callback is destroyed.
  * @event_type: a character string indicating the type of events for which
  *            notification is requested.  Format is
  *            EventClass:major_type:minor_type:detail
@@ -463,7 +465,8 @@ listener_entry_free (EventListenerEntry *e)
  **/
 gboolean
 atspi_event_listener_register (AtspiEventListener *listener,
-				             const gchar              *event_type)
+				             const gchar              *event_type,
+				             GError **error)
 {
   /* TODO: Keep track of which events have been registered, so that we
  * deregister all of them when the event listener is destroyed */
@@ -471,7 +474,7 @@ atspi_event_listener_register (AtspiEventListener *listener,
   return atspi_event_listener_register_from_callback (listener->callback,
                                                       listener->user_data,
                                                       listener->cb_destroyed,
-                                                      event_type);
+                                                      event_type, error);
 }
 
 /**
@@ -488,11 +491,12 @@ gboolean
 atspi_event_listener_register_from_callback (AtspiEventListenerCB callback,
 				             void *user_data,
 				             GDestroyNotify callback_destroyed,
-				             const gchar              *event_type)
+				             const gchar              *event_type,
+				             GError **error)
 {
   EventListenerEntry *e;
   char *matchrule;
-  DBusError error;
+  DBusError d_error;
   GList *new_list;
   DBusMessage *message, *reply;
 
@@ -501,6 +505,12 @@ atspi_event_listener_register_from_callback (AtspiEventListenerCB callback,
       return FALSE;
     }
 
+  if (!event_type)
+  {
+    g_warning (_("called atspi_event_listener_register_from_callback with a NULL event_type"));
+    return FALSE;
+  }
+
   e = g_new (EventListenerEntry, 1);
   if (!e) return FALSE;
   e->callback = callback;
@@ -520,23 +530,25 @@ atspi_event_listener_register_from_callback (AtspiEventListenerCB callback,
     return FALSE;
   }
   event_listeners = new_list;
-  dbus_error_init (&error);
-  dbus_bus_add_match (_atspi_bus(), matchrule, &error);
-  if (error.message)
+  dbus_error_init (&d_error);
+  dbus_bus_add_match (_atspi_bus(), matchrule, &d_error);
+  if (d_error.message)
   {
-    g_warning ("Atspi: Adding match: %s", error.message);
+    g_warning ("Atspi: Adding match: %s", d_error.message);
+    /* TODO: Set error */
   }
 
-  dbus_error_init (&error);
+  dbus_error_init (&d_error);
   message = dbus_message_new_method_call (atspi_bus_registry,
 	atspi_path_registry,
 	atspi_interface_registry,
 	"RegisterEvent");
   if (!message)
-    return;
+    return FALSE;
   dbus_message_append_args (message, DBUS_TYPE_STRING, &event_type, DBUS_TYPE_INVALID);
-  reply = _atspi_dbus_send_with_reply_and_block (message);
-  dbus_message_unref (reply);
+  reply = _atspi_dbus_send_with_reply_and_block (message, error);
+  if (reply)
+    dbus_message_unref (reply);
 
   return TRUE;
 }
@@ -559,9 +571,12 @@ atspi_event_listener_register_from_callback (AtspiEventListenerCB callback,
 gboolean
 atspi_event_listener_register_no_data (AtspiEventListenerSimpleCB callback,
 				 GDestroyNotify callback_destroyed,
-				 const gchar              *event_type)
+				 const gchar              *event_type,
+				 GError **error)
 {
-  return atspi_event_listener_register_from_callback (remove_datum, callback, callback_destroyed, event_type);
+  return atspi_event_listener_register_from_callback (remove_datum, callback,
+                                                      callback_destroyed,
+                                                      event_type, error);
 }
 
 static gboolean
@@ -585,9 +600,12 @@ is_superset (const gchar *super, const gchar *sub)
  **/
 gboolean
 atspi_event_listener_deregister (AtspiEventListener *listener,
-				               const gchar              *event_type)
+				               const gchar              *event_type,
+				               GError **error)
 {
-  atspi_event_listener_deregister_from_callback (listener->callback, listener->user_data, event_type);
+  return atspi_event_listener_deregister_from_callback (listener->callback,
+                                                        listener->user_data,
+                                                        event_type, error);
 }
 
 /**
@@ -606,7 +624,8 @@ atspi_event_listener_deregister (AtspiEventListener *listener,
 gboolean
 atspi_event_listener_deregister_from_callback (AtspiEventListenerCB callback,
 				               void *user_data,
-				               const gchar              *event_type)
+				               const gchar              *event_type,
+				               GError **error)
 {
   char *category, *name, *detail, *matchrule;
   GList *l;
@@ -629,20 +648,24 @@ atspi_event_listener_deregister_from_callback (AtspiEventListenerCB callback,
         is_superset (name, e->name) &&
         is_superset (detail, e->detail))
     {
-      DBusError error;
+      gboolean need_replace;
+      DBusError d_error;
       DBusMessage *message, *reply;
+      need_replace = (l == event_listeners);
       l = g_list_remove (l, e);
-      dbus_error_init (&error);
-      dbus_bus_remove_match (_atspi_bus(), matchrule, &error);
-      dbus_error_init (&error);
+      if (need_replace)
+        event_listeners = l;
+      dbus_error_init (&d_error);
+      dbus_bus_remove_match (_atspi_bus(), matchrule, &d_error);
+      dbus_error_init (&d_error);
       message = dbus_message_new_method_call (atspi_bus_registry,
 	    atspi_path_registry,
 	    atspi_interface_registry,
 	    "RegisterEvent");
       if (!message)
-      return;
+      return FALSE;
       dbus_message_append_args (message, DBUS_TYPE_STRING, &event_type, DBUS_TYPE_INVALID);
-      reply = _atspi_dbus_send_with_reply_and_block (message);
+      reply = _atspi_dbus_send_with_reply_and_block (message, error);
       dbus_message_unref (reply);
 
       listener_entry_free (e);
@@ -670,10 +693,12 @@ atspi_event_listener_deregister_from_callback (AtspiEventListenerCB callback,
  **/
 gboolean
 atspi_event_listener_deregister_no_data (AtspiEventListenerSimpleCB callback,
-				   const gchar              *event_type)
+				   const gchar              *event_type,
+				   GError **error)
 {
   return atspi_event_listener_deregister_from_callback (remove_datum, callback,
-                                                        event_type);
+                                                        event_type,
+                                                        error);
 }
 
 static AtspiEvent *
@@ -738,6 +763,7 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   char *detail = NULL;
   const char *category = dbus_message_get_interface (message);
   const char *member = dbus_message_get_member (message);
+  const char *signature = dbus_message_get_signature (message);
   gchar *name;
   gchar *converted_type;
   DBusMessageIter iter, iter_variant;
@@ -746,6 +772,12 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   dbus_int32_t detail1, detail2;
   char *p;
 
+  if (strcmp (signature, "siiv(so)") != 0)
+  {
+    g_warning (_("Got invalid signature %s for signal %s from interface %s\n"), signature, member, category);
+    return DBUS_HANDLER_RESULT_HANDLED;
+  }
+
   memset (&e, 0, sizeof (e));
 
   if (category)
@@ -758,15 +790,11 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
     }
     category++;
   }
-  g_return_val_if_fail (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_STRING, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
   dbus_message_iter_get_basic (&iter, &detail);
   dbus_message_iter_next (&iter);
-  /* TODO: Return error indicating invalid arguments  in next line */
-  g_return_val_if_fail (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_INT32, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
   dbus_message_iter_get_basic (&iter, &detail1);
   e.detail1 = detail1;
   dbus_message_iter_next (&iter);
-  g_return_val_if_fail (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_INT32, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
   dbus_message_iter_get_basic (&iter, &detail2);
   e.detail2 = detail2;
   dbus_message_iter_next (&iter);
@@ -784,6 +812,16 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
       converted_type = p;
     }
   }
+  else if (detail [0] == '\0')
+  {
+    p = g_strconcat (converted_type, ":",  NULL);
+    if (p)
+    {
+      g_free (converted_type);
+      converted_type = p;
+    }
+  }
+
   if (detail[0] != '\0')
   {
     p = g_strconcat (converted_type, ":", detail, NULL);
@@ -813,6 +851,7 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
 	accessible = _atspi_dbus_return_accessible_from_iter (&iter_variant);
 	g_value_init (&e.any_data, ATSPI_TYPE_ACCESSIBLE);
 	g_value_set_instance (&e.any_data, accessible);
+	g_object_unref (accessible);	/* value now owns it */
       }
       break;
     }
@@ -826,7 +865,6 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   default:
     break;
   }
-  _atspi_send_event (&e);
 
   if (!strncmp (e.type, "object:children-changed", 23))
   {
@@ -841,6 +879,8 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
     cache_process_state_changed (&e);
   }
 
+  _atspi_send_event (&e);
+
   g_free (converted_type);
   g_free (name);
   g_free (detail);
