@@ -75,7 +75,7 @@ G_DEFINE_BOXED_TYPE (AtspiTextRange, atspi_text_range, atspi_text_range_copy,
 gint
 atspi_text_get_character_count (AtspiText *obj, GError **error)
 {
-  dbus_int32_t retval;
+  dbus_int32_t retval = 0;
 
   g_return_val_if_fail (obj != NULL, -1);
 
@@ -647,11 +647,11 @@ atspi_text_get_bounded_ranges (AtspiText *obj,
 gint
 atspi_text_get_n_selections (AtspiText *obj, GError **error)
 {
-  dbus_int32_t retval;
+  dbus_int32_t retval = 0;
 
   g_return_val_if_fail (obj != NULL, -1);
 
-  _atspi_dbus_call (obj, atspi_interface_text, "GetNSelections", error, "i", &retval);
+  _atspi_dbus_call (obj, atspi_interface_text, "GetNSelections", error, "=>i", &retval);
 
   return retval;
 }
