@@ -35,6 +35,7 @@
 #include <sys/time.h>
 
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/extensions/XTest.h>
 #include <X11/XKBlib.h>
 #define XK_MISCELLANY
@@ -56,6 +57,7 @@
 #include "de-types.h"
 #include "de-marshaller.h"
 #include "display.h"
+#include "event-source.h"
 
 #include "deviceeventcontroller.h"
 #include "reentrant-list.h"
@@ -408,8 +410,8 @@ static void emit(SpiDEController *controller, const char *interface, const char
   dbus_message_iter_init_append (signal, &iter);
 
   dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &minor);
-  dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &a1);
-  dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &a2);
+  dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &a1);
+  dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &a2);
   dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, "i", &iter_variant);
       dbus_message_iter_append_basic (&iter_variant, DBUS_TYPE_INT32, &nil);
   dbus_message_iter_close_container (&iter, &iter_variant);
@@ -421,6 +423,7 @@ static void emit(SpiDEController *controller, const char *interface, const char
   dbus_message_iter_close_container (&iter, &iter_struct);
 
   dbus_connection_send (controller->bus, signal, NULL);
+  dbus_message_unref (signal);
 }
 
 static gboolean
@@ -836,6 +839,8 @@ spi_key_listener_data_free (DEControllerKeyListener *key_listener)
 {
   keylist_free(key_listener->keys);
   if (key_listener->mode) g_free(key_listener->mode);
+  g_free (key_listener->listener.bus_name);
+  g_free (key_listener->listener.path);
   g_free (key_listener);
 }
 
@@ -856,10 +861,13 @@ spi_listener_clone_free (DEControllerListener *clone)
 static void
 spi_dec_listener_free (DEControllerListener    *listener)
 {
-  g_free (listener->bus_name);
-  g_free (listener->path);
   if (listener->type == SPI_DEVICE_TYPE_KBD) 
     spi_key_listener_data_free ((DEControllerKeyListener *) listener);
+  else
+  {
+    g_free (listener->bus_name);
+    g_free (listener->path);
+  }
 }
 
 static void
@@ -999,7 +1007,7 @@ spi_controller_register_device_listener (SpiDEController      *controller,
         {
           have_mouse_listener = TRUE;
           if (!have_mouse_event_listener)
-            g_timeout_add (100, spi_dec_poll_mouse_idle, controller->registry);
+            g_timeout_add (100, spi_dec_poll_mouse_idle, controller);
         }
       spi_dbus_add_disconnect_match (controller->bus, listener->bus_name);
       break;
@@ -1033,6 +1041,7 @@ send_and_allow_reentry (DBusConnection *bus, DBusMessage *message, int timeout,
       if (!dbus_connection_read_write_dispatch (bus, timeout))
         return NULL;
     }
+    dbus_pending_call_unref (pending);
     return reply;
 }
 static gboolean
@@ -1904,6 +1913,7 @@ impl_register_keystroke_listener (DBusConnection *bus,
   dbus_bool_t ret;
   DBusMessage *reply = NULL;
   char *keystring;
+  char *sig;
 
   dbus_message_iter_init(message, &iter);
   // TODO: verify type signature
@@ -1923,7 +1933,8 @@ impl_register_keystroke_listener (DBusConnection *bus,
   dbus_message_iter_next(&iter);
   dbus_message_iter_get_basic(&iter, &mask);
   dbus_message_iter_next(&iter);
-  if (!strcmp (dbus_message_iter_get_signature (&iter), "u"))
+  sig = dbus_message_iter_get_signature (&iter);
+  if (sig && !strcmp (sig, "u"))
     dbus_message_iter_get_basic(&iter, &type);
   else
   {
@@ -1937,6 +1948,7 @@ impl_register_keystroke_listener (DBusConnection *bus,
     }
     dbus_message_iter_next (&iter_array);
   }
+  dbus_free (sig);
   dbus_message_iter_next(&iter);
   mode = (Accessibility_EventListenerMode *)g_malloc(sizeof(Accessibility_EventListenerMode));
   if (mode)
@@ -1948,6 +1960,7 @@ impl_register_keystroke_listener (DBusConnection *bus,
 	   dbus_message_get_sender(message), path, (unsigned long) mask);
 #endif
   dec_listener = spi_dec_key_listener_new (dbus_message_get_sender(message), path, keys, mask, type, mode);
+  g_free (mode);
   ret = spi_controller_register_device_listener (
 	  controller, (DEControllerListener *) dec_listener);
   reply = dbus_message_new_method_return (message);
@@ -2959,6 +2972,7 @@ spi_registry_dec_new (SpiRegistry *reg, DBusConnection *bus)
   SpiDEController *dec = g_object_new (SPI_DEVICE_EVENT_CONTROLLER_TYPE, NULL);
 
   dec->registry = g_object_ref (reg);
+  reg->dec = g_object_ref (dec);
   dec->bus = bus;
 
   dbus_connection_register_object_path (bus, SPI_DBUS_PATH_DEC, &dec_vtable, dec);
@@ -2975,7 +2989,7 @@ spi_device_event_controller_start_poll_mouse (SpiRegistry *registry)
     {
       have_mouse_event_listener = TRUE;
       if (!have_mouse_listener)
-      g_timeout_add (100, spi_dec_poll_mouse_idle, registry);
+      g_timeout_add (100, spi_dec_poll_mouse_idle, registry->dec);
     }
 }
 
